//
//  StreamMonitorPacket.m
//  GlowWorm
//
//  Created by Curtis Jones on 2007.02.26.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "StreamMonitorPacket.h"
#import <Common.h>
#import <Logger.h>
#import <GlowWorm.h>


@implementation StreamMonitorPacket

- (id)initWithOffset:(uint32_t)offs andPckt:(GlowWormPckt *)aPckt
{
	self = [super init];
	
	NSData * data = [aPckt data];
	
	mPckt  = [aPckt retain];
	mVert  = 0;
	mHead  = [NSMutableString stringWithCapacity:100];
	mByte  = [NSMutableString stringWithCapacity:([data length] / 16) * 8];
	mData  = [NSMutableString stringWithCapacity:[data length] * 4];
	mAscii = [NSMutableString stringWithCapacity:[data length] + ([data length] / 16)];
	
	NSDate * date = [NSDate dateWithTimeIntervalSince1970:[[aPckt time] unsignedLongLongValue] / 1000000.];
	
	{
		NSString * datestr = [GlowWorm dateToString:date];
		NSString * addrtx = nil;
		NSString * addrrx = nil;
		
		if (GWS_CONN_DATA_INCOMING == [[aPckt direction] unsignedLongValue]) {
			addrtx = [aPckt hrRemAddrPort];
			addrrx = [aPckt hrLocAddrPort];
		}
		else {
			addrrx = [aPckt hrRemAddrPort];
			addrtx = [aPckt hrLocAddrPort];
		}
		
		[mHead appendFormat:@"data: dec %lu, hex 0x%x\n", (uint32_t)[data length], (uint32_t)[data length]];
		[mHead appendFormat:@"date: %@\n", datestr];
		[mHead appendFormat:@"addr: %@ -> %@", addrtx, addrrx];
		
//	[mHead appendFormat:@"bytes: %lu, %@ -> %@, %@", (uint32_t)[data length], addrtx, addrrx, datestr];
	}
	
	{
		int i = 0, leng = [data length];
		u_char * bytes = (u_char *)[data bytes];
		
		for (i = 1; i <= leng; ++i) {
			if (bytes[i-1] >= 32 && bytes[i-1] <= 126)
				[mAscii appendFormat:@"%c", bytes[i-1]];
			else
				[mAscii appendFormat:@"."];
			
			if (i != leng && 0 == i % 16) {
				[mData appendFormat:@"%02X\n", bytes[i-1]];
				[mByte appendFormat:@"%06x\n", offs];
				[mAscii appendFormat:@"\n"];
				offs += 16;
			}
			else if (i != leng && 0 == i % 8)
				[mData appendFormat:@"%02X   ", bytes[i-1]];
			else if (i == leng) {
				[mData appendFormat:@"%02X", bytes[i-1]];
				[mByte appendFormat:@"%06x", offs];
				offs += leng % 16;
			}
			else
				[mData appendFormat:@"%02X ", bytes[i-1]];
		}
	}
	
	return self;
}

- (id)initWithOffset:(uint32_t)offs data:(NSData *)data
{
	self = [super init];
	
	mVert  = 0;
	mByte  = [NSMutableString stringWithCapacity:([data length] / 16) * 8];
	mData  = [NSMutableString stringWithCapacity:[data length] * 4];
	mAscii = [NSMutableString stringWithCapacity:[data length] + ([data length] / 16)];
	
	{
		int i = 0, leng = [data length];
		u_char * bytes = (u_char *)[data bytes];
		
		for (i = 1; i <= leng; ++i) {
			if (bytes[i-1] >= 32 && bytes[i-1] <= 126)
				[mAscii appendFormat:@"%c", bytes[i-1]];
			else
				[mAscii appendFormat:@"."];
			
			if (i != leng && 0 == i % 16) {
				[mData appendFormat:@"%02X\n", bytes[i-1]];
				[mByte appendFormat:@"%06x\n", offs];
				[mAscii appendFormat:@"\n"];
				offs += 16;
			}
			else if (i != leng && 0 == i % 8)
				[mData appendFormat:@"%02X   ", bytes[i-1]];
			else if (i == leng) {
				[mData appendFormat:@"%02X", bytes[i-1]];
				[mByte appendFormat:@"%06x", offs];
				offs += leng % 16;
			}
			else
				[mData appendFormat:@"%02X ", bytes[i-1]];
		}
	}
	
	return self;
}

- (void)dealloc
{
	[mPckt release];
	
	[mHead release];
	[mByte release];
	[mData release];
	[mAscii release];
	
	[mHeadTxt release];
	[mByteTxt release];
	[mDataTxt release];
	[mAsciiTxt release];
	
	[super dealloc];
}

- (int)height
{
	return [[mHeadTxt cell] cellSize].height + [[mDataTxt cell] cellSize].height;
}

- (void)setupViews:(uint32_t)vertOffs view:(NSView *)aView
{
	NSColor * bgcolor = nil;
	
	if (GWS_CONN_DATA_INCOMING == [[mPckt direction] unsignedLongValue])
		bgcolor = [NSColor colorWithDeviceRed:0.69 green:0.96 blue:0.69 alpha:1.];
	else
		bgcolor = [NSColor colorWithDeviceRed:0.96 green:0.69 blue:0.69 alpha:1.];
	
	mHeadTxt = [[NSTextField alloc] init];
	[mHeadTxt setFont:[NSFont fontWithName:@"Andale Mono" size:9.0]];
	[mHeadTxt setBezeled:FALSE];
	[mHeadTxt setDrawsBackground:TRUE];
	[mHeadTxt setBackgroundColor:[NSColor whiteColor]];
	[mHeadTxt setFocusRingType:NSFocusRingTypeNone];
	[mHeadTxt setEditable:FALSE];
	[mHeadTxt setSelectable:FALSE];
	[mHeadTxt setStringValue:mHead];
	[mHeadTxt setAutoresizingMask:NSViewNotSizable];
	
	mByteTxt = [[NSTextField alloc] init];
	[mByteTxt setFont:[NSFont fontWithName:@"Andale Mono" size:9.0]];
	[mByteTxt setBezeled:FALSE];
	[mByteTxt setDrawsBackground:TRUE];
	[mByteTxt setBackgroundColor:[NSColor whiteColor]];
	[mByteTxt setFocusRingType:NSFocusRingTypeNone];
	[mByteTxt setEditable:FALSE];
	[mByteTxt setSelectable:FALSE];
	[mByteTxt setStringValue:mByte];
	[mByteTxt setAutoresizingMask:NSViewNotSizable];
	
	mDataTxt = [[NSTextField alloc] init];
	[mDataTxt setFont:[NSFont fontWithName:@"Andale Mono" size:9.0]];
	[mDataTxt setBezeled:FALSE];
	[mDataTxt setDrawsBackground:TRUE];
	[mDataTxt setBackgroundColor:bgcolor];
	[mDataTxt setFocusRingType:NSFocusRingTypeNone];
	[mDataTxt setEditable:FALSE];
	[mDataTxt setSelectable:TRUE];
	[mDataTxt setStringValue:mData];
	[mDataTxt setAutoresizingMask:NSViewNotSizable];
	
	mAsciiTxt = [[NSTextField alloc] init];
	[mAsciiTxt setFont:[NSFont fontWithName:@"Andale Mono" size:9.0]];
	[mAsciiTxt setBezeled:FALSE];
	[mAsciiTxt setDrawsBackground:TRUE];
	[mAsciiTxt setBackgroundColor:[NSColor whiteColor]];
	[mAsciiTxt setFocusRingType:NSFocusRingTypeNone];
	[mAsciiTxt setEditable:FALSE];
	[mAsciiTxt setSelectable:TRUE];
	[mAsciiTxt setStringValue:mAscii];
	[mAsciiTxt setAutoresizingMask:NSViewNotSizable];
	
	
	
	
	
	int pcktHeight = [[mHeadTxt cell] cellSize].height + [[mDataTxt cell] cellSize].height;
	
	// resize the view and push all of the existing subviews back up to
	// the top of the view.
	//
	if (vertOffs + pcktHeight > [aView frame].size.height) {
		NSRect visif = [aView visibleRect];
		
		NSRect frame = [aView frame];
		uint32_t oldheight = frame.size.height;
		frame.size.height = vertOffs + pcktHeight;
		[aView setFrame:frame];
		uint32_t difheight = frame.size.height - oldheight;
		
		uint32_t i=0;
		NSArray * subviews = [aView subviews];
		
		for (i = 0; i < [subviews count]; ++i) {
			NSView * nsview = [subviews objectAtIndex:i];
			
			frame = [nsview frame];
			frame.origin.y += difheight;
			
			[nsview setFrameOrigin:frame.origin];
		}
		
		{
			if (visif.origin.y > 0)
				visif.origin.y += difheight;
			
			[aView scrollRectToVisible:visif];
			[aView setNeedsDisplayInRect:visif];
		}
		
		vertOffs = [aView frame].size.height - pcktHeight;
	}
	
	
	
	
	{
		NSSize size;
		size.width = 250;
		size.height = [[mHeadTxt cell] cellSize].height;
		[mHeadTxt setFrameSize:size];
		
		NSPoint origin;
		origin.x = 45;
		origin.y = [aView frame].size.height - (vertOffs + size.height);
		[mHeadTxt setFrameOrigin:origin];
		
		[aView addSubview:mHeadTxt];
	}
	
	
	
	{
		NSSize size;
		size.width = 40;
		size.height = [[mByteTxt cell] cellSize].height;
		[mByteTxt setFrameSize:size];
		
		NSPoint origin;
		origin.x = 0;
		origin.y = [aView frame].size.height - (vertOffs + pcktHeight);
		[mByteTxt setFrameOrigin:origin];
		
		[aView addSubview:mByteTxt];
	}
	
	
	
	{
		NSSize size;
		size.width = 100;
		size.height = [[mAsciiTxt cell] cellSize].height;
		[mAsciiTxt setFrameSize:size];
		
		NSPoint origin;
		origin.x = 300;
		origin.y = [aView frame].size.height - (vertOffs + pcktHeight);
		[mAsciiTxt setFrameOrigin:origin];
		
		[aView addSubview:mAsciiTxt];
	}
	
	
	
	{
		NSSize size;
		size.width = 250;
		size.height = [[mDataTxt cell] cellSize].height;
		[mDataTxt setFrameSize:size];
		
		NSPoint origin;
		origin.x = 45;
		origin.y = [aView frame].size.height - (vertOffs + pcktHeight);
		[mDataTxt setFrameOrigin:origin];
		
		[aView addSubview:mDataTxt];
	}
	
}

@end
